/*
 *  Copyright (C) 2008, 2009 Juan Pedro Bolívar Puente
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package game;

/**
 * Base de todo juego de cartas. Implementa un control básico sobre el estado
 * del juego que puede ser útil para evitar código redundante en las clases
 * derivadas.
 * 
 * @author raskolnikov
 */
public abstract class Game extends GameSubject
{
	/**
	 * Comienza una partida.
	 * 
	 * @throws GameException
	 *             si la partida ya ha comenzado o debido a violaciones lógicas
	 *             inherentes al juego implementado.
	 */
	public void start () throws GameException
	{
		if (!m_started) {
			m_started = true;
		} else
			throw new GameException ("Game started already");
	}
	
	/**
	 * Finaliza una partida.
	 * 
	 * @throws GameException
	 *             si la partida no ha comenzado u otro tipo de violación lógica
	 *             dependiente de la implementación.
	 */
	public void stop () throws GameException
	{
		if (m_started)
			m_started = false;
		else
			throw new GameException ("Game not in course");		
	}
	
	/**
	 * Juega una partida hasta que termine por cauce natural. Notar que aun así
	 * es un requisito lógico invocar un ciclo stop () start () para poder jugar
	 * otra partida.
	 * 
	 * @throws GameException
	 *             si existen violaciones de la lógica del juego o la partida no
	 *             ha comenzado.
	 */
	public void play () throws GameException
	{
		while (!playRound ());
	}
	
	/**
	 * Juega una ronda de la partida. Notar que entendemos por ronda como "un
	 * ciclo de juego en el que un mismo jugador no tiene que hacer más de una
	 * decisión." Por conveniencia cuando no sea un requisito este nivel de
	 * control, puede utilizarse play ()
	 * 
	 * @return Si tras esta ronda la partida ha terminado.
	 * @throws GameException
	 *             Si existen violaciones de la lógica del juego o la partida no
	 *             ha comenzado.
	 * @see #play()
	 */
	public boolean playRound () throws GameException
	{
		if (!m_started)
			throw new GameException ("Game not in course");
		return false;
	}
	
	/**
	 * Un jugador se une a la partida.
	 * 
	 * @param p El jugador que se une a la partida.
	 * @throws GameException
	 *             si el jugador se intenta unir a una partida que ya ha
	 *             comenzado y otro tipo de error lógico dependiente de la
	 *             implementación.
	 */
	public void join (Player p) throws GameException
	{
		if (m_started)
			throw new GameException ("Game started already");
		else
			p.setGame (this);
	}
	
	/**
	 * Devuelve si la partida está en curso.
	 * @return Si la partida ya ha comenzado.
	 */
	public boolean isStarted ()
	{
		return m_started;
	}
	
	/**
	 * Devuelve la apuesta mínima que debe hacer un jugador en el estado actual
	 * de la partida.
	 * 
	 * @return La apuesta mínima.
	 */
	public double getMinBet ()
	{
		return 0.0;
	}
	
	/** Si la partida ha comenzado */
	private boolean m_started = false;
}
